#------------------------------------------------------------------------------
savepdf<-function(file, width=15, height=15) {
#------------------------------------------------------------------------------
# Save a plot in a PDF file

  fname <- paste(file,".pdf",sep="")
  pdf(fname, width=width/2.54, height=height/2.54,
      pointsize=10)
  par(mgp=c(2.2,0.3,0), tcl=-0.4, mar=c(3.3,3.6,0.5,0.9))
}


#------------------------------------------------------------------------------
plotZ<-function(datafr,z,eMin,eMax,barn=1e6,yScale=1.1,cexScale=0.9,logxy="x") {
#------------------------------------------------------------------------------
# Function to plot the cross section for selected Z

# Select the data sample corresponding to the current Z, with energy in the desired range
df<-subset(datafr, Z==z & E>eMin & E<eMax)
	
# Associate an element name to the current atomic number
element<-c(elementName[z])
	
# Rescale cross sections to proper units to look nicely in the plots
df$exp<-df$exp/barn
df$err<-df$err/barn
df$original<-df$original/barn
df$improved<-df$improved/barn
		
# Show the current element name and Z on top of the legend of the plot
title = bquote(paste(.(element)," Z = ",.(z)))
	
# Define the horizontal and vertical scale of the plot
yMax<-c(yScale*max(df$exp))
xMax<-c(1.05*max(df$E))
xMin<-c(0.95*min(df$E))
unit<-(" (b)") 
if (barn == 1e6) unit<-(" (Mb)")
else if (barn == 1000) unit<-(" (Kb)")
			
# Plot original calculated cross sections as a function of energy
plot(df$E, df$original, xlim=c(xMin,xMax), ylim=c(0.001,yMax), pch=4, col="red", 
		las=1, tck=0.01, 
		xlab="E (keV)", ylab=paste("Cross section ",unit), 
		log=logxy, 
		cex=1, cex.axis=0.85,cex.lab=1,cex.main=0.1)
		
# Add tick marks on all sides of the plot
axis(3,labels=FALSE,tck=0.01,cex=cexScale)	
axis(4,labels=FALSE,tck=0.01,cex=cexScale)
	
# Add improved calculated cross sections to the current plot
points(df$E,df$improved,pch=8,col="cyan",cex=cexScale)

# Initialize legend
legendNames<-vector()
legendPch<-vector()
legendCol<-vector()
bgRef<-vector()
legendBg<-vector()
nRef<-c(0)

# Treat bibliographical references as factors
df$ref=as.factor(df$ref)

# Identify the experimental data samples for the current Z
refNames<-levels(df$ref)
nRef<-length(refNames)
    
# Associate a different symbol to each experimental data sample; plot them in black, 
# then in grey (if all black solid symbols are exhausted due to many experimental data samples )
blackRef<-rep("black",nRef)
pchRef<-seq(15,15+nRef-1)
bgRef<-rep("darkgrey",nRef)
    	
# Experimental points plotted according to bibliographical source: points and error bars
points(df$E,df$exp,pch=pchRef[df$ref],bg=bgRef[df$ref],col=blackRef[df$ref],cex=cexScale)
segments(df$E,df$exp+df$err,df$E,df$exp-df$err)

# Information for the legend
legendNames<-c(legendNames,refNames)
legendPch<-c(legendPch,pchRef)
legendCol<-c(legendCol,blackRef)
legendBg<-c(legendBg,bgRef)	

# Prepare vectors for the legend: plot the symbol of a model in the legend only if associated data are present
    
if (max(df$improved) > 0) 
   {
	legendNames<-c(legendNames, "improved")
   	legendPch<-c(legendPch,8)
   	legendCol<-c(legendCol,"cyan")
   }
    	
if (max(df$original) > 0) 
   {
    	legendNames<-c(legendNames, "original")
    	legendPch<-c(legendPch,4)
    	legendCol<-c(legendCol,"red")
   }

# Plot the legend
legend("topright", legendNames, title=paste(element, ", Z=", z,sep=""), pch=legendPch, col=legendCol, pt.bg=legendBg, y.intersp=0.9, cex=0.9, bty="n")
    
return()
}

#-----------------------------------------------------------------------------
plotPDF<-function(datafr,eMin,eMax){
#-----------------------------------------------------------------------------
# Produce PDF 

# Z = 10
dummy<-savepdf("Z-10")
dummy<-plotZ(datafr, 10, 0.1, 0.25, 1e6, 1.1, 0.8, "x")
dev.off()

# Z = 18
dummy<-savepdf("Z-18")
dummy<-plotZ(datafr, 18, 0.1, 0.25, 1e6, 1.1, 0.8, "x")
dev.off()
}

#==============================================================================
# MAIN
#==============================================================================

# ---- Graphics settings
par(xaxs="r")
par(yaxs="r")
colo<-colors()
#
# ---- Element symbols
elementName<-c(
	"H", "He", "Li", "Be", "B", "C", "N", "O", "F", "Ne", "Na", "Mg", "Al", "Si", "P", "S", "Cl", "Ar", 
	"K", "Ca", "Sc", "Ti", "V", "Cr", "Mn", "Fe", "Co", "Ni", "Cu", "Zn", "Ga", "Ge", "As", "Se", "Br", 		
	"Kr", "Rb", "Sr", "Y", "Zr", "Nb", "Mo", "Tc", "Ru", "Rh", "Pd", "Ag", "Cd", "In", "Sn", "Sb", "Te", 				
	"I", "Xe", "Cs", "Ba", "La", "Ce", "Pr", "Nd", "Pm", "Sm", "Eu", "Gd", "Tb", "Dy", "Ho", "Er", "Tm", 				
	"Yb", "Lu", "Hf", "Ta", "W", "Re", "Os", "Ir", "Pt", "Au", "Hg", "Tl", "Pb", "Bi", "Po", "At", "Rn", 				
	"Fr", "Ra", "Ac", "Th", "Pa", "U")

# Column names in the data frame
colName<-c("ref", "Z", "E", "exp", "err", "original", "improved")
nColumns<-length(colName)

# Load data frame (ntuple) containing experimental and calculated cross section data
dir<-Sys.getenv("APCDIR")
fileName<-paste(dir,"/dataAPC/all_photoelectricAPC.txt",sep="")
datafr<-read.table(fileName,header=FALSE,col.names=colName,as.is=c(1:nColumns))

# Dump the structure of the data frame to verify that it has been correctly loaded
#str(datafr)
						
# ---- Add a column with merged Z+ref, to be treated as a factor
Zref<-paste(datafr$Z,"-",datafr$ref,sep="")
datafr<-cbind(datafr,Zref)

message("Data loaded, start plotting")

# Divide the plotting window into two vertical zones
par(mfrow=c(1,2))
# Graphics prameters settings (margins, distance of axis labels etc.)
par(mgp=c(2.2,0.3,0))
par(oma=(c(0,0,0,0)))
par(mar=c(3.3,3.6,0.3,0.5))

# Plot experimental and calculated cross sections for neon
dummy<-plotZ(datafr,10, 0.1, 0.25)

# Plot experimental and calculated cross sections for argon
dummy<-plotZ(datafr, 18, 0.1, 0.25)

# Make PDF files
message("Make PDF files with plots")
dummy<-plotPDF(datafr,0.1,99999999)

message("End plotting")

#==============================================================================
